@@ -3,7 +3,7 @@
   bignum.c -
 
   $Author: matz $
-  $Date: 2001/08/24 06:36:14 $
+  $Date: 2001/11/01 05:09:51 $
   created at: Fri Jun 10 00:48:55 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -591,8 +591,7 @@ rb_big_eq(x, y)
       case T_BIGNUM:
 	break;
       case T_FLOAT:
-	y = dbl2big(RFLOAT(y)->value);
-	break;
+	return (rb_big2dbl(x) == RFLOAT(y)->value)?Qtrue:Qfalse;
       default:
 	return Qfalse;
     }
@@ -1317,9 +1316,15 @@ rb_big_aref(x, y)
     VALUE x, y;
 {
     BDIGIT *xds;
-    int shift = NUM2INT(y);
+    int shift;
     int s1, s2;
 
+    if (TYPE(y) == T_BIGNUM) {
+	if (!RBIGNUM(y)->sign || RBIGNUM(x)->sign)
+	    return INT2FIX(0);
+	return INT2FIX(1);
+    }
+    shift = NUM2INT(y);
     if (shift < 0) return INT2FIX(0);
     s1 = shift/BITSPERDIG;
     s2 = shift%BITSPERDIG;
