@@ -2,8 +2,8 @@
 
   error.c -
 
-  $Author: matz $
-  $Date: 2001/02/02 11:39:51 $
+  $Author: knu $
+  $Date: 2001/09/03 05:18:02 $
   created at: Mon Aug  9 16:11:34 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -468,7 +468,7 @@ static const syserr_index_entry syserr_index[]= {
 static VALUE *syserr_list;
 #endif
 
-#if !defined NT && !defined sys_nerr
+#if !defined(NT) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(sys_nerr)
 extern int sys_nerr;
 #endif
 
