@@ -423,8 +423,8 @@ freeDevice()
 
 /* ruby function: getpty */
 static VALUE
-pty_getpty(self, shell)
-    VALUE self, shell;
+pty_getpty(self, command)
+    VALUE self, command;
 {
     VALUE res;
     struct pty_info info;
@@ -442,24 +442,28 @@ pty_getpty(self, shell)
     OBJSETUP(wport, rb_cFile, T_FILE);
     MakeOpenFile(wport, wfptr);
 
-    establishShell(RSTRING(shell)->ptr,&info);
+    if (TYPE(command) == T_ARRAY)
+	command = rb_ary_join(command,rb_str_new2(" "));
+    Check_SafeStr(command);
+
+    establishShell(RSTRING(command)->ptr,&info);
 
     rfptr->mode = rb_io_mode_flags("r");
     rfptr->f = fdopen(info.fd, "r");
-    rfptr->path = strdup(RSTRING(shell)->ptr);
+    rfptr->path = strdup(RSTRING(command)->ptr);
 
     wfptr->mode = rb_io_mode_flags("w");
     wfptr->f = fdopen(dup(info.fd), "w");
-    wfptr->path = strdup(RSTRING(shell)->ptr);
+    wfptr->path = strdup(RSTRING(command)->ptr);
 
-    res = rb_ary_new2(2);
+    res = rb_ary_new2(4);
     rb_ary_store(res,0,(VALUE)rport);
     rb_ary_store(res,1,(VALUE)wport);
     rb_ary_store(res,2,INT2FIX(info.child_pid));
+    rb_ary_store(res,3,rb_str_new2(SlaveName));
 
     if (rb_block_given_p()) {
-	rb_yield((VALUE)res);
-	reset_signal_action();
+	rb_ensure(rb_yield, (VALUE)res, (VALUE (*)())reset_signal_action, Qnil);
 	return Qnil;
     }
     else {
