@@ -3,7 +3,7 @@
   gc.c -
 
   $Author: matz $
-  $Date: 2001/05/01 08:45:11 $
+  $Date: 2001/08/06 03:10:23 $
   created at: Tue Oct  5 09:44:46 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -138,11 +138,12 @@ ruby_xrealloc(ptr, size)
     if (!mem) {
 	rb_gc();
 	RUBY_CRITICAL(mem = realloc(ptr, size));
-	if (!mem)
+	if (!mem) {
 	    if (size >= 50 * 1024 * 1024) {
 		rb_raise(rb_eNoMemError, "tried to re-allocate too big memory");
 	    }
 	    mem_error("failed to allocate memory(realloc)");
+	}
     }
 
     return mem;
@@ -262,7 +263,10 @@ static RVALUE **heaps;
 static int heaps_length = 0;
 static int heaps_used   = 0;
 
-#define HEAP_SLOTS 10000
+#define HEAP_MIN_SLOTS 10000
+static int *heaps_limits;
+static int heap_slots = HEAP_MIN_SLOTS;
+
 #define FREE_MIN  4096
 
 static RVALUE *himem, *lomem;
@@ -279,13 +283,29 @@ add_heap()
 			(RVALUE**)realloc(heaps, heaps_length*sizeof(RVALUE*)):
 			(RVALUE**)malloc(heaps_length*sizeof(RVALUE*)));
 	if (heaps == 0) mem_error("heaps: can't alloc memory");
+	RUBY_CRITICAL(heaps_limits = (heaps_used>0)?
+			(int*)realloc(heaps_limits, heaps_length*sizeof(int)):
+			(int*)malloc(heaps_length*sizeof(int)));
+	if (heaps_limits == 0) mem_error("heaps_limits: can't alloc memory");
     }
 
-    RUBY_CRITICAL(p = heaps[heaps_used++] = (RVALUE*)malloc(sizeof(RVALUE)*HEAP_SLOTS));
-    if (p == 0) mem_error("add_heap: can't alloc memory");
-    pend = p + HEAP_SLOTS;
+    for (;;) {
+	RUBY_CRITICAL(p = heaps[heaps_used] = (RVALUE*)malloc(sizeof(RVALUE)*heap_slots));
+	heaps_limits[heaps_used] = heap_slots;
+	if (p == 0) {
+	    if (heap_slots == HEAP_MIN_SLOTS) {
+		mem_error("add_heap: can't alloc memory");
+	    }
+	    heap_slots = HEAP_MIN_SLOTS;
+	    continue;
+	}
+	break;
+    }
+    pend = p + heap_slots;
     if (lomem == 0 || lomem > p) lomem = p;
     if (himem < pend) himem = pend;
+    heaps_used++;
+    heap_slots *= 2;
 
     while (p < pend) {
 	p->as.free.flag = 0;
@@ -341,8 +361,8 @@ is_pointer_to_heap(ptr)
     /* check if p looks like a pointer */
     for (i=0; i < heaps_used; i++) {
 	heap_org = heaps[i];
-	if (heap_org <= p && p < heap_org + HEAP_SLOTS
-	    && ((((char*)p)-((char*)heap_org))%sizeof(RVALUE)) == 0)
+	if (heap_org <= p && p < heap_org + heaps_limits[i] &&
+	    ((((char*)p)-((char*)heap_org))%sizeof(RVALUE)) == 0)
 	    return Qtrue;
     }
     return Qfalse;
@@ -516,6 +536,8 @@ rb_gc_mark(ptr)
 	  case NODE_DEFINED:
 	  case NODE_MATCH:
 	  case NODE_RETURN:
+	  case NODE_BREAK:
+	  case NODE_NEXT:
 	  case NODE_YIELD:
 	  case NODE_COLON2:
 	  case NODE_ARGS:
@@ -543,8 +565,6 @@ rb_gc_mark(ptr)
 	  case NODE_BACK_REF:
 	  case NODE_ALIAS:
 	  case NODE_VALIAS:
-	  case NODE_BREAK:
-	  case NODE_NEXT:
 	  case NODE_REDO:
 	  case NODE_RETRY:
 	  case NODE_UNDEF:
@@ -680,7 +700,7 @@ gc_sweep()
     if (ruby_in_compile) {
 	/* should not reclaim nodes during compilation */
 	for (i = 0; i < used; i++) {
-	    p = heaps[i]; pend = p + HEAP_SLOTS;
+	    p = heaps[i]; pend = p + heaps_limits[i];
 	    while (p < pend) {
 		if (!(p->as.basic.flags&FL_MARK) && BUILTIN_TYPE(p) == T_NODE)
 		    rb_gc_mark(p);
@@ -695,7 +715,7 @@ gc_sweep()
     for (i = 0; i < used; i++) {
 	int n = 0;
 
-	p = heaps[i]; pend = p + HEAP_SLOTS;
+	p = heaps[i]; pend = p + heaps_limits[i];
 	while (p < pend) {
 	    if (!(p->as.basic.flags & FL_MARK)) {
 		if (p->as.basic.flags) {
@@ -898,11 +918,11 @@ rb_gc_mark_frame(frame)
 #if defined(__human68k__) || defined(DJGPP)
 #if defined(__human68k__)
 typedef unsigned long rb_jmp_buf[8];
-__asm__ (".even
-_rb_setjmp:
-	move.l	4(sp),a0
-	movem.l	d3-d7/a3-a5,(a0)
-	moveq.l	#0,d0
+__asm__ (".even\n\
+_rb_setjmp:\n\
+	move.l	4(sp),a0\n\
+	movem.l	d3-d7/a3-a5,(a0)\n\
+	moveq.l	#0,d0\n\
 	rts");
 #ifdef setjmp
 #undef setjmp
@@ -910,19 +930,19 @@ _rb_setjmp:
 #else
 #if defined(DJGPP)
 typedef unsigned long rb_jmp_buf[6];
-__asm__ (".align 4
-_rb_setjmp:
-	pushl	%ebp
-	movl	%esp,%ebp
-	movl	8(%ebp),%ebp
-	movl	%eax,(%ebp)
-	movl	%ebx,4(%ebp)
-	movl	%ecx,8(%ebp)
-	movl	%edx,12(%ebp)
-	movl	%esi,16(%ebp)
-	movl	%edi,20(%ebp)
-	popl	%ebp
-	xorl	%eax,%eax
+__asm__ (".align 4\n\
+_rb_setjmp:\n\
+	pushl	%ebp\n\
+	movl	%esp,%ebp\n\
+	movl	8(%ebp),%ebp\n\
+	movl	%eax,(%ebp)\n\
+	movl	%ebx,4(%ebp)\n\
+	movl	%ecx,8(%ebp)\n\
+	movl	%edx,12(%ebp)\n\
+	movl	%esi,16(%ebp)\n\
+	movl	%edi,20(%ebp)\n\
+	popl	%ebp\n\
+	xorl	%eax,%eax\n\
 	ret");
 #endif
 #endif
@@ -943,7 +963,7 @@ rb_gc()
     alloca(0);
 # define STACK_END (&stack_end)
 #else
-# if defined(__GNUC__) && (defined(__i386__) || defined(__m68k__))
+# if defined(__GNUC__) && (defined(__i386__) || defined(__mc68000__))
     VALUE *stack_end = __builtin_frame_address(0);
 # else
     VALUE *stack_end = alloca(1);
@@ -1023,8 +1043,6 @@ Init_stack(addr)
 #if defined(__human68k__)
     extern void *_SEND;
     rb_gc_stack_start = _SEND;
-#elif defined(__GNUC__) && (defined(__i386__) || defined(__m68k__))
-    rb_gc_stack_start = __builtin_frame_address(2);
 #else
     VALUE start;
 
@@ -1051,7 +1069,7 @@ os_live_obj()
     for (i = 0; i < heaps_used; i++) {
 	RVALUE *p, *pend;
 
-	p = heaps[i]; pend = p + HEAP_SLOTS;
+	p = heaps[i]; pend = p + heaps_limits[i];
 	for (;p < pend; p++) {
 	    if (p->as.basic.flags) {
 		switch (TYPE(p)) {
@@ -1084,7 +1102,7 @@ os_obj_of(of)
     for (i = 0; i < heaps_used; i++) {
 	RVALUE *p, *pend;
 
-	p = heaps[i]; pend = p + HEAP_SLOTS;
+	p = heaps[i]; pend = p + heaps_limits[i];
 	for (;p < pend; p++) {
 	    if (p->as.basic.flags) {
 		switch (TYPE(p)) {
@@ -1253,7 +1271,7 @@ rb_gc_call_finalizer_at_exit()
 	    }
 	}
 	for (i = 0; i < heaps_used; i++) {
-	    p = heaps[i]; pend = p + HEAP_SLOTS;
+	    p = heaps[i]; pend = p + heaps_limits[i];
 	    while (p < pend) {
 		if (FL_TEST(p, FL_FINALIZE)) {
 		    FL_UNSET(p, FL_FINALIZE);
@@ -1266,7 +1284,7 @@ rb_gc_call_finalizer_at_exit()
     }
     /* run data object's finaliers */
     for (i = 0; i < heaps_used; i++) {
-	p = heaps[i]; pend = p + HEAP_SLOTS;
+	p = heaps[i]; pend = p + heaps_limits[i];
 	while (p < pend) {
 	    if (BUILTIN_TYPE(p) == T_DATA &&
 		DATA_PTR(p) && RANY(p)->as.data.dfree) {
