@@ -3,7 +3,7 @@
   io.c -
 
   $Author: matz $
-  $Date: 2001/09/05 06:53:27 $
+  $Date: 2001/12/21 09:20:53 $
   created at: Fri Oct 15 18:08:59 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -476,7 +476,11 @@ io_fread(ptr, len, f)
 	if (c == EOF) {
 	    if (ferror(f)) {
 		if (errno == EINTR) continue;
-		rb_sys_fail(0);
+		if (errno == EAGAIN) return len - n;
+#if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN
+		if (errno == EWOULDBLOCK) return len - n;
+#endif
+		return 0;
 	    }
 	    *ptr = '\0';
 	    break;
@@ -1097,7 +1101,7 @@ rb_io_closed(io)
     return (fptr->f || fptr->f2)?Qfalse:Qtrue;
 }
 
-VALUE
+static VALUE
 rb_io_close_read(io)
     VALUE io;
 {
@@ -2130,18 +2134,17 @@ rb_io_puts(argc, argv, out)
 	return Qnil;
     }
     for (i=0; i<argc; i++) {
-	switch (TYPE(argv[i])) {
-	  case T_NIL:
+	if (NIL_P(argv[i])) {
 	    line = rb_str_new2("nil");
-	    break;
-	  case T_ARRAY:
-	    rb_protect_inspect(io_puts_ary, argv[i], out);
-	    continue;
-	  default:
-	    line = argv[i];
-	    break;
 	}
-	line = rb_obj_as_string(line);
+	else {
+	    line = rb_check_convert_type(argv[i], T_ARRAY, "Array", "to_ary");
+	    if (!NIL_P(line)) {
+		rb_protect_inspect(io_puts_ary, line, out);
+		continue;
+	    }
+	    line = rb_obj_as_string(argv[i]);
+	}
 	rb_io_write(out, line);
 	if (RSTRING(line)->ptr[RSTRING(line)->len-1] != '\n') {
 	    rb_io_write(out, rb_default_rs);
