@@ -10,10 +10,10 @@
 for k,v in ENV
   next unless /^[a-zA-Z][_a-zA-Z0-9]*/ =~ k
   eval <<EOS
-  $#{k} = %q!#{v}!
+  $#{k} = v
   trace_var "$#{k}", proc{|v|
-    ENV[%q!#{k}!] = v; 
-    $#{k} = %q!#{v}!
+    ENV[%q!#{k}!] = v
+    $#{k} = v
     if v == nil
       untrace_var "$#{k}"
     end
