@@ -1,80 +1,41 @@
 =begin
 
-= net/protocol.rb version 1.1.29
+= net/protocol.rb version 1.1.37
 
-written by Minero Aoki <aamine@dp.u-netsurf.ne.jp>
+Copyright (c) 1999-2001 Yukihiro Matsumoto
 
-This program is free software.
-You can distribute/modify this program under
-the terms of the Ruby Distribute License.
+written & maintained by Minero Aoki <aamine@loveruby.net>
 
-Japanese version of this document is in "net" full package.
-You can get it from RAA
-(Ruby Application Archive: http://www.ruby-lang.org/en/raa.html).
+This program is free software. You can re-distribute and/or
+modify this program under the same terms as Ruby itself,
+Ruby Distribute License or GNU General Public License.
 
-
-== Net::Protocol
-
-the abstract class for Internet protocol
-
-=== Super Class
-
-Object
-
-=== Class Methods
-
-: new( address = 'localhost', port = nil )
-  This method Creates a new protocol object.
-
-: start( address = 'localhost', port = nil, *protoargs )
-: start( address = 'localhost', port = nil, *protoargs ) {|proto| .... }
-  This method creates a new Protocol object and opens a session.
-  equals to Net::Protocol.new( address, port ).start( *protoargs )
-
-: Proxy( address, port )
-  This method creates a proxy class of its protocol.
-  Arguments are address/port of proxy host.
-
-=== Methods
-
-: address
-  the address of connecting server (FQDN).
-
-: port
-  connecting port number
-
-: start( *args )
-: start( *args ) {|proto| .... }
-  This method starts protocol. If protocol was already started,
-  do nothing and returns false.
-
-  '*args' are specified in subclasses.
-
-  When is called with block, gives Protocol object to block and
-  close session when block finished.
-
-: finish
-  This method ends protocol. If you call this method before protocol starts,
-  it only return false without doing anything.
-
-: active?
-  true if session have been started
+NOTE: You can find Japanese version of this document in
+the doc/net directory of the standard ruby interpreter package.
 
 =end
 
 require 'socket'
+require 'timeout'
 
 
 module Net
 
+  module NetPrivate
+  end
+
+  def self.net_private( &block )
+    ::Net::NetPrivate.module_eval( &block )
+  end
+
+
   class Protocol
 
-    Version = '1.1.29'
-
+    Version = '1.1.37'
 
     class << self
 
-      def start( address = 'localhost', port = nil, *args )
+      def start( address, port = nil, *args )
         instance = new( address, port )
 
         if block_given? then
@@ -85,38 +46,6 @@ module Net
         end
       end
 
-      def Proxy( p_addr, p_port = nil )
-        p_port ||= self.port
-        klass = Class.new( self )
-        klass.module_eval %-
-
-          def initialize( addr, port )
-            @proxyaddr = '#{p_addr}'
-            @proxyport = '#{p_port}'
-            super @proxyaddr, @proxyport
-            @address = addr
-            @port    = port
-          end
-
-          def connect( addr = nil, port = nil )
-            super @proxyaddr, @proxyport
-          end
-          private :connect
-            
-          attr_reader :proxyaddr, :proxyport
-        -
-        def klass.proxy?
-          true
-        end
-
-        klass
-      end
-
-      def proxy?
-        false
-      end
-
-
       private
 
       def protocol_param( name, val )
@@ -131,13 +60,17 @@ module Net
 
 
     #
-    # sub-class requirements
+    # --- Configuration Staffs for Sub Classes ---
     #
-    # protocol_param command_type
-    # protocol_param port
+    #   protocol_param port
+    #   protocol_param command_type
+    #   protocol_param socket_type   (optional)
     #
-    # private method do_start  (optional)
-    # private method do_finish (optional)
+    #   private method do_start      (optional)
+    #   private method do_finish     (optional)
+    #
+    #   private method on_connect    (optional)
+    #   private method on_disconnect (optional)
     #
 
     protocol_param :port,         'nil'
@@ -145,70 +78,117 @@ module Net
     protocol_param :socket_type,  '::Net::NetPrivate::Socket'
 
 
-    def initialize( addr = nil, port = nil )
-      @address = addr || 'localhost'
+    def initialize( addr, port = nil )
+      @address = addr
       @port    = port || type.port
 
-      @active  = false
-      @pipe    = nil
-
       @command = nil
       @socket  = nil
-    end
 
-    attr_reader :address, :port,
-                :command, :socket
-
-    def inspect
-      "#<#{type} #{address}:#{port} open=#{active?}>"
-    end
-
-
-    def start( *args )
-      return false if active?
-
-      begin
-        connect
-        do_start( *args )
-        @active = true
-        yield self if block_given?
-      ensure
-        finish if block_given?
-      end
-    end
-
-    def finish
-      return false unless active?
-
-      do_finish
-      disconnect
       @active = false
-      true
+
+      @open_timeout = nil
+      @read_timeout = nil
+
+      @dout = nil
     end
 
+    attr_reader :address
+    attr_reader :port
+
+    attr_reader :command
+    attr_reader :socket
+
+    attr_accessor :open_timeout
+    attr_accessor :read_timeout
+
     def active?
       @active
     end
 
-    def set_pipe( arg )   # un-documented
-      @pipe = arg
+    def set_debug_output( arg )   # un-documented
+      @dout = arg
     end
 
+    alias set_pipe set_debug_output
+
+    def inspect
+      "#<#{type} #{address}:#{port} open=#{active?}>"
+    end
+
+    #
+    # open session
+    #
+
+    def start( *args )
+      active? and raise IOError, 'protocol has been opened already'
+
+      if block_given? then
+        begin
+          _start args
+          yield self
+        ensure
+          finish if active?
+        end
+      else
+        _start args
+      end
+      nil
+    end
 
     private
 
+    def _start( args )
+      connect
+      do_start( *args )
+      @active = true
+    end
+
+    def connect
+      conn_socket @address, @port
+      on_connect
+      conn_command @socket
+    end
+
+    def re_connect
+      @socket.reopen @open_timeout
+      on_connect
+    end
+
+    def conn_socket( addr, port )
+      @socket = type.socket_type.open(
+              addr, port, @open_timeout, @read_timeout, @dout )
+    end
+
+    def conn_command( sock )
+      @command = type.command_type.new( sock )
+    end
+
+    def on_connect
+    end
 
     def do_start
     end
 
-    def do_finish
-      @command.quit
+    #
+    # close session
+    #
+
+    public
+
+    def finish
+      active? or raise IOError, 'already closed protocol'
+
+      do_finish if @command and not @command.critical?
+      disconnect
+      @active = false
+      nil
     end
 
+    private
 
-    def connect( addr = @address, port = @port )
-      @socket  = type.socket_type.open( addr, port, @pipe )
-      @command = type.command_type.new( @socket )
+    def do_finish
+      @command.quit
     end
 
     def disconnect
@@ -216,7 +196,11 @@ module Net
       if @socket and not @socket.closed? then
         @socket.close
       end
-      @socket  = nil
+      @socket = nil
+      on_disconnect
+    end
+
+    def on_disconnect
     end
     
   end
@@ -224,13 +208,13 @@ module Net
   Session = Protocol
 
 
-
+  net_private {
 
   class Response
 
-    def initialize( ctype, cno, msg )
+    def initialize( ctype, code, msg )
       @code_type = ctype
-      @code      = cno
+      @code      = code
       @message   = msg
       super()
     end
@@ -242,12 +226,14 @@ module Net
       "#<#{type} #{code}>"
     end
 
-    def error!( data = nil )
-      raise code_type.error_type.new( code + ' ' + Net.quote(msg), data )
+    def error!
+      raise code_type.error_type.new( code + ' ' + Net.quote(msg), self )
     end
 
   end
 
+  }
+
 
   class ProtocolError          < StandardError; end
   class ProtoSyntaxError       < ProtocolError; end
@@ -261,12 +247,13 @@ module Net
 
   class ProtocolError
   
-    def initialize( msg, data )
+    def initialize( msg, resp )
       super msg
-      @data = data
+      @response = resp
     end
 
-    attr :data
+    attr :response
+    alias data response
 
     def inspect
       "#<#{type}>"
@@ -306,6 +293,7 @@ module Net
   end
   
   ReplyCode       = Code.new( [], ProtoUnknownError )
+  InformationCode = ReplyCode.mkchild( ProtoUnknownError )
   SuccessCode     = ReplyCode.mkchild( ProtoUnknownError )
   ContinueCode    = ReplyCode.mkchild( ProtoUnknownError )
   ErrorCode       = ReplyCode.mkchild( ProtocolError )
@@ -318,8 +306,7 @@ module Net
 
 
 
-  module NetPrivate
-
+  net_private {
 
   class WriteAdapter
 
@@ -335,7 +322,11 @@ module Net
     def write( str )
       @sock.__send__ @mid, str
     end
-    alias << write
+
+    def <<( str )
+      @sock.__send__ @mid, str
+      self
+    end
   
   end
 
@@ -350,12 +341,31 @@ module Net
     end
 
     def <<( str )
-      @block.call str
+      callblock( str, &@block ) if @block
+    end
+
+    private
+
+    def callblock( str )
+      begin
+        user_break = true
+        yield str
+        user_break = false
+      rescue Exception
+        user_break = false
+        raise
+      ensure
+        if user_break then
+          @block = nil
+          return   # stop break
+        end
+      end
     end
   
   end
 
 
+
   class Command
 
     def initialize( sock )
@@ -392,24 +402,36 @@ module Net
       rep.error!
     end
 
-    def getok( line, ok = SuccessCode )
+    def getok( line, expect = SuccessCode )
       @socket.writeline line
-      check_reply ok
+      check_reply expect
     end
 
 
-    def critical
-      return if @critical
-      @critical = true
-      r = yield
-      @critical = false
-      r
-    end
+    #
+    # error handle
+    #
+
+    public
 
     def critical?
       @critical
     end
 
+    def error_ok
+      @critical = false
+    end
+
+
+    private
+
+    def critical
+      @critical = true
+      ret = yield
+      @critical = false
+      ret
+    end
+
     def begin_critical
       ret = @critical
       @critical = true
@@ -425,22 +447,30 @@ module Net
 
   class Socket
 
-    def initialize( addr, port, pipe = nil )
+    def initialize( addr, port, otime = nil, rtime = nil, dout = nil )
       @addr = addr
       @port = port
-      @pipe = pipe
-      @prepipe = nil
 
-      @closed  = true
-      @ipaddr  = ''
+      @read_timeout = rtime
+
+      @debugout = dout
+
+      @socket = nil
       @sending = ''
       @buffer  = ''
 
-      @socket = TCPsocket.new( addr, port )
-      @closed = false
-      @ipaddr = @socket.addr[3]
+      connect otime
+      D 'opened'
     end
 
+    def connect( otime )
+      D "opening connection to #{@addr}..."
+      timeout( otime ) {
+        @socket = TCPsocket.new( @addr, @port )
+      }
+    end
+    private :connect
+
     attr :pipe, true
 
     class << self
@@ -448,152 +478,178 @@ module Net
     end
 
     def inspect
-      "#<#{type} open=#{!@closed}>"
+      "#<#{type} #{closed? ? 'closed' : 'opened'}>"
     end
 
-    def reopen
-      unless closed? then
-        @socket.close
-        @buffer = ''
-      end
-      @socket = TCPsocket.new( @addr, @port )
+    def reopen( otime = nil )
+      D 'reopening...'
+      close
+      connect otime
+      D 'reopened'
     end
 
     attr :socket, true
 
     def close
-      @socket.close
-      @closed = true
+      if @socket then
+        @socket.close
+        D 'closed'
+      else
+        D 'close call for already closed socket'
+      end
+      @socket = nil
+      @buffer = ''
     end
 
     def closed?
-      @closed
+      not @socket
     end
 
     def address
       @addr.dup
     end
+
     alias addr address
 
     attr_reader :port
 
     def ip_address
-      @ipaddr.dup
+      @socket or return ''
+      @socket.addr[3]
     end
+
     alias ipaddr ip_address
 
     attr_reader :sending
 
 
-    CRLF    = "\r\n"
+    #
+    # read
+    #
 
-    def read( len, dest = '' )
-      @pipe << "reading #{len} bytes...\n" if @pipe; pipeoff
+    public
+
+    CRLF = "\r\n"
+
+    def read( len, dest = '', igneof = false )
+      D_off "reading #{len} bytes..."
 
       rsize = 0
-      while rsize + @buffer.size < len do
-        rsize += writeinto( dest, @buffer.size )
-        fill_rbuf
+      begin
+        while rsize + @buffer.size < len do
+          rsize += rbuf_moveto( dest, @buffer.size )
+          rbuf_fill
+        end
+        rbuf_moveto dest, len - rsize
+      rescue EOFError
+        raise unless igneof
       end
-      writeinto( dest, len - rsize )
 
-      @pipe << "read #{len} bytes\n" if pipeon
+      D_on "read #{len} bytes"
       dest
     end
 
-
     def read_all( dest = '' )
-      @pipe << "reading all...\n" if @pipe; pipeoff
+      D_off 'reading all...'
 
       rsize = 0
       begin
         while true do
-          rsize += writeinto( dest, @buffer.size )
-          fill_rbuf
+          rsize += rbuf_moveto( dest, @buffer.size )
+          rbuf_fill
         end
       rescue EOFError
         ;
       end
 
-      @pipe << "read #{rsize} bytes\n" if pipeon
+      D_on "read #{rsize} bytes"
       dest
     end
 
-
-    def readuntil( target )
-      while true do
-        idx = @buffer.index( target )
-        break if idx
-        fill_rbuf
-      end
-
+    def readuntil( target, igneof = false )
       dest = ''
-      writeinto( dest, idx + target.size )
+      begin
+        while true do
+          idx = @buffer.index( target )
+          break if idx
+          rbuf_fill
+        end
+        rbuf_moveto dest, idx + target.size
+      rescue EOFError
+        raise unless igneof
+        rbuf_moveto dest, @buffer.size
+      end
       dest
     end
-
         
     def readline
-      ret = readuntil( "\r\n" )
+      ret = readuntil( "\n" )
       ret.chop!
       ret
     end
 
-
     def read_pendstr( dest )
-      @pipe << "reading text...\n" if @pipe; pipeoff
+      D_off 'reading text...'
 
       rsize = 0
-
-      while (str = readuntil( "\r\n" )) != ".\r\n" do
+      while (str = readuntil("\r\n")) != ".\r\n" do
         rsize += str.size
         str.gsub!( /\A\./, '' )
         dest << str
       end
 
-      @pipe << "read #{rsize} bytes\n" if pipeon
+      D_on "read #{rsize} bytes"
       dest
     end
 
-
+    # private use only (can not handle 'break')
     def read_pendlist
-      @pipe << "reading list...\n" if @pipe; pipeoff
+    #  D_off 'reading list...'
 
-      arr = []
       str = nil
-
-      while (str = readuntil( "\r\n" )) != ".\r\n" do
+      i = 0
+      while (str = readuntil("\r\n")) != ".\r\n" do
+        i += 1
         str.chop!
-        arr.push str
-        yield str if block_given?
+        yield str
       end
 
-      @pipe << "read #{arr.size} lines\n" if pipeon
-      arr
+    #  D_on "read #{i} items"
     end
 
 
     private
 
 
-    READ_BLOCK = 1024 * 8
+    READ_SIZE = 1024 * 4
 
-    def fill_rbuf
-      @buffer << @socket.sysread( READ_BLOCK )
+    def rbuf_fill
+      unless IO.select [@socket], nil, nil, @read_timeout then
+        on_read_timeout
+      end
+      @buffer << @socket.sysread( READ_SIZE )
     end
 
-    def writeinto( dest, len )
+    def on_read_timeout
+      raise TimeoutError, "socket read timeout (#{@read_timeout} sec)"
+    end
+
+    def rbuf_moveto( dest, len )
       bsi = @buffer.size
-      dest << @buffer[ 0, len ]
+      s = @buffer[ 0, len ]
+      dest << s
       @buffer = @buffer[ len, bsi - len ]
 
-      @pipe << %{read  "#{Net.quote dest}"\n} if @pipe
+      @debugout << %<read  "#{Net.quote s}"\n> if @debugout
       len
     end
 
 
-    public
+    #
+    # write interfece
+    #
 
+    public
 
     def write( str )
       writing {
@@ -601,19 +657,16 @@ module Net
       }
     end
 
-
     def writeline( str )
       writing {
-        do_write str
-        do_write "\r\n"
+        do_write str + "\r\n"
       }
     end
 
-
     def write_bin( src, block )
       writing {
         if block then
-          block.call WriteAdapter.new( self, :do_write )
+          block.call ::Net::NetPrivate::WriteAdapter.new( self, :do_write )
         else
           src.each do |bin|
             do_write bin
@@ -622,19 +675,18 @@ module Net
       }
     end
 
-
     def write_pendstr( src, block )
-      @pipe << "writing text from #{src.type}\n" if @pipe; pipeoff
+      D_off "writing text from #{src.type}"
 
       wsize = use_each_crlf_line {
         if block then
-          block.call WriteAdapter.new( self, :wpend_in )
+          block.call ::Net::NetPrivate::WriteAdapter.new( self, :wpend_in )
         else
           wpend_in src
         end
       }
 
-      @pipe << "wrote #{wsize} bytes text\n" if pipeon
+      D_on "wrote #{wsize} bytes text"
       wsize
     end
 
@@ -681,17 +733,17 @@ module Net
         beg = 0
         buf = @wbuf
         while buf.index( /\n|\r\n|\r/, beg ) do
-          m = $~
+          m = Regexp.last_match
           if m.begin(0) == buf.size - 1 and buf[-1] == ?\r then
             # "...\r" : can follow "\n..."
             break
           end
-          str = buf[ beg, m.begin(0) - beg ]
+          str = buf[ beg ... m.begin(0) ]
           str.concat "\r\n"
           yield str
           beg = m.end(0)
         end
-        @wbuf = buf[ beg, buf.size - beg ]
+        @wbuf = buf[ beg ... buf.size ]
       end
     end
 
@@ -721,6 +773,7 @@ module Net
             yield
           end
         end
+        yield unless @wbuf.empty?
       end
     end
 
@@ -731,17 +784,17 @@ module Net
 
       yield
 
-      if @pipe then
-        @pipe << 'write "'
-        @pipe << @sending
-        @pipe << "\"\n"
+      if @debugout then
+        @debugout << 'write "'
+        @debugout << @sending
+        @debugout << "\"\n"
       end
       @socket.flush
       @writtensize
     end
 
     def do_write( arg )
-      if @pipe or @sending.size < 128 then
+      if @debugout or @sending.size < 128 then
         @sending << Net.quote( arg )
       else
         @sending << '...' unless @sending[-1] == ?.
@@ -753,22 +806,25 @@ module Net
     end
 
 
-    def pipeoff
-      @prepipe = @pipe
-      @pipe = nil
-      @prepipe
+    def D_off( msg )
+      D msg
+      @savedo, @debugout = @debugout, nil
     end
 
-    def pipeon
-      @pipe = @prepipe
-      @prepipe = nil
-      @pipe
+    def D_on( msg )
+      @debugout = @savedo
+      D msg
+    end
+
+    def D( msg )
+      @debugout or return
+      @debugout << msg
+      @debugout << "\n"
     end
 
   end
 
-
-  end   # module Net::NetPrivate
+  }
 
 
   def Net.quote( str )
