@@ -2,8 +2,8 @@
 
   object.c -
 
-  $Author: nobu $
-  $Date: 2001/08/11 14:10:23 $
+  $Author: matz $
+  $Date: 2001/12/18 08:40:31 $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -72,18 +72,26 @@ rb_obj_id(obj)
     return (VALUE)((long)obj|FIXNUM_FLAG);
 }
 
-static VALUE
-rb_obj_type(obj)
-    VALUE obj;
+VALUE
+rb_class_real(cl)
+    VALUE cl;
 {
-    VALUE cl = CLASS_OF(obj);
-
+    if (TYPE(cl) == T_ICLASS) {
+	cl = RBASIC(cl)->klass;
+    }
     while (FL_TEST(cl, FL_SINGLETON) || TYPE(cl) == T_ICLASS) {
 	cl = RCLASS(cl)->super;
     }
     return cl;
 }
 
+VALUE
+rb_obj_class(obj)
+    VALUE obj;
+{
+    return rb_class_real(CLASS_OF(obj));
+}
+
 VALUE
 rb_obj_clone(obj)
     VALUE obj;
@@ -113,7 +121,7 @@ rb_obj_dup(obj)
 	rb_raise(rb_eTypeError, "dupulicated object must be same type");
     }
     if (!SPECIAL_CONST_P(dup)) {
-	OBJSETUP(dup, rb_obj_type(obj), BUILTIN_TYPE(obj));
+	OBJSETUP(dup, rb_obj_class(obj), BUILTIN_TYPE(obj));
 	OBJ_INFECT(dup, obj);
 	if (FL_TEST(obj, FL_EXIVAR)) {
 	    FL_SET(dup, FL_EXIVAR);
@@ -241,7 +249,7 @@ rb_obj_is_instance_of(obj, c)
 	rb_raise(rb_eTypeError, "class or module required");
     }
 
-    if (rb_obj_type(obj) == c) return Qtrue;
+    if (rb_obj_class(obj) == c) return Qtrue;
     return Qfalse;
 }
 
@@ -359,13 +367,6 @@ nil_inspect(obj)
     return rb_str_new2("nil");
 }
 
-static VALUE
-nil_type(obj)
-    VALUE obj;
-{
-    return rb_cNilClass;
-}
-
 #ifdef NIL_PLUS
 static VALUE
 nil_plus(x, y)
@@ -402,13 +403,6 @@ true_to_s(obj)
     return rb_str_new2("true");
 }
 
-static VALUE
-true_type(obj)
-    VALUE obj;
-{
-    return rb_cTrueClass;
-}
-
 static VALUE
 true_and(obj, obj2)
     VALUE obj, obj2;
@@ -437,13 +431,6 @@ false_to_s(obj)
     return rb_str_new2("false");
 }
 
-static VALUE
-false_type(obj)
-    VALUE obj;
-{
-    return rb_cFalseClass;
-}
-
 static VALUE
 false_and(obj, obj2)
     VALUE obj, obj2;
@@ -489,13 +476,6 @@ rb_obj_alloc(klass)
     return (VALUE)obj;
 }
 
-static VALUE
-sym_type(sym)
-    VALUE sym;
-{
-    return rb_cSymbol;
-}
-
 static VALUE
 sym_to_i(sym)
     VALUE sym;
@@ -873,6 +853,27 @@ rb_convert_type(val, type, tname, method)
     return val;
 }
 
+VALUE
+rb_check_convert_type(val, type, tname, method)
+    VALUE val;
+    int type;
+    const char *tname, *method;
+{
+    struct arg_to arg1, arg2;
+
+    if (TYPE(val) == type) return val;
+    arg1.val = arg2.val = val;
+    arg1.s = method;
+    arg2.s = tname;
+    val = rb_rescue2(to_type, (VALUE)&arg1, 0, 0,
+		     rb_eStandardError, rb_eNameError, 0);
+    if (!NIL_P(val) && TYPE(val) != type) {
+	rb_raise(rb_eTypeError, "%s#%s should return %s",
+		 rb_class2name(CLASS_OF(arg1.val)), method, tname);
+    }
+    return val;
+}
+
 static VALUE
 rb_to_integer(val, method)
     VALUE val;
@@ -1156,8 +1157,8 @@ Init_Object()
     rb_define_method(rb_mKernel, "hash", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "id", rb_obj_id, 0);
     rb_define_method(rb_mKernel, "__id__", rb_obj_id, 0);
-    rb_define_method(rb_mKernel, "type", rb_obj_type, 0);
-    rb_define_method(rb_mKernel, "class", rb_obj_type, 0);
+    rb_define_method(rb_mKernel, "type", rb_obj_class, 0);
+    rb_define_method(rb_mKernel, "class", rb_obj_class, 0);
 
     rb_define_method(rb_mKernel, "clone", rb_obj_clone, 0);
     rb_define_method(rb_mKernel, "dup", rb_obj_dup, 0);
@@ -1196,7 +1197,6 @@ Init_Object()
     rb_define_global_function("Array", rb_f_array, 1);
 
     rb_cNilClass = rb_define_class("NilClass", rb_cObject);
-    rb_define_method(rb_cNilClass, "type", nil_type, 0);
     rb_define_method(rb_cNilClass, "to_i", nil_to_i, 0);
     rb_define_method(rb_cNilClass, "to_s", nil_to_s, 0);
     rb_define_method(rb_cNilClass, "to_a", nil_to_a, 0);
@@ -1211,7 +1211,6 @@ Init_Object()
 
     rb_cSymbol = rb_define_class("Symbol", rb_cObject);
     rb_undef_method(CLASS_OF(rb_cSymbol), "new");
-    rb_define_method(rb_cSymbol, "type", sym_type, 0);
     rb_define_method(rb_cSymbol, "to_i", sym_to_i, 0);
     rb_define_method(rb_cSymbol, "to_int", sym_to_i, 0);
     rb_define_method(rb_cSymbol, "inspect", sym_inspect, 0);
@@ -1268,7 +1267,6 @@ Init_Object()
 
     rb_cTrueClass = rb_define_class("TrueClass", rb_cObject);
     rb_define_method(rb_cTrueClass, "to_s", true_to_s, 0);
-    rb_define_method(rb_cTrueClass, "type", true_type, 0);
     rb_define_method(rb_cTrueClass, "&", true_and, 1);
     rb_define_method(rb_cTrueClass, "|", true_or, 1);
     rb_define_method(rb_cTrueClass, "^", true_xor, 1);
@@ -1277,7 +1275,6 @@ Init_Object()
 
     rb_cFalseClass = rb_define_class("FalseClass", rb_cObject);
     rb_define_method(rb_cFalseClass, "to_s", false_to_s, 0);
-    rb_define_method(rb_cFalseClass, "type", false_type, 0);
     rb_define_method(rb_cFalseClass, "&", false_and, 1);
     rb_define_method(rb_cFalseClass, "|", false_or, 1);
     rb_define_method(rb_cFalseClass, "^", false_xor, 1);
