@@ -636,6 +636,13 @@ rb_reg_search(re, str, pos, reverse)
     if (NIL_P(match) || FL_TEST(match, MATCH_BUSY)) {
 	match = match_alloc();
     }
+    else {
+	if (rb_safe_level() >= 3) 
+	    OBJ_TAINT(match);
+	else
+	    FL_UNSET(match, FL_TAINT);
+    }
+
     re_copy_registers(RMATCH(match)->regs, &regs);
     RMATCH(match)->str = rb_str_new4(str);
     rb_backref_set(match);
@@ -652,7 +659,11 @@ rb_reg_nth_defined(nth, match)
 {
     if (NIL_P(match)) return Qnil;
     if (nth >= RMATCH(match)->regs->num_regs) {
-	return Qfalse;
+	return Qnil;
+    }
+    if (nth < 0) {
+	nth += RMATCH(match)->regs->num_regs;
+	if (nth <= 0) return Qnil;
     }
     if (RMATCH(match)->BEG(nth) == -1) return Qfalse;
     return Qtrue;
@@ -670,6 +681,10 @@ rb_reg_nth_match(nth, match)
     if (nth >= RMATCH(match)->regs->num_regs) {
 	return Qnil;
     }
+    if (nth < 0) {
+	nth += RMATCH(match)->regs->num_regs;
+	if (nth <= 0) return Qnil;
+    }
     start = RMATCH(match)->BEG(nth);
     if (start == -1) return Qnil;
     end = RMATCH(match)->END(nth);
@@ -752,11 +767,16 @@ match_to_a(match)
     VALUE ary = rb_ary_new2(regs->num_regs);
     char *ptr = RSTRING(RMATCH(match)->str)->ptr;
     int i;
-
+    int taint = OBJ_TAINTED(match);
+    
     for (i=0; i<regs->num_regs; i++) {
-	if (regs->beg[i] == -1) rb_ary_push(ary, Qnil);
-	else rb_ary_push(ary, rb_str_new(ptr+regs->beg[i],
-				   regs->end[i]-regs->beg[i]));
+	if (regs->beg[i] == -1) {
+	    rb_ary_push(ary, Qnil);
+	} else {
+	    VALUE str = rb_str_new(ptr+regs->beg[i], regs->end[i]-regs->beg[i]);
+	    if (taint) OBJ_TAINT(str);
+	    rb_ary_push(ary, str);
+	}
     }
     return ary;
 }
@@ -1074,7 +1094,7 @@ rb_reg_s_quote(argc, argv)
     }
     kcode_reset_option();
     rb_str_resize(tmp, t - RSTRING(tmp)->ptr);
-
+    OBJ_INFECT(tmp, str);
     return tmp;
 }
 
@@ -1145,7 +1165,7 @@ rb_reg_clone(re)
     clone->ptr = 0; clone->len = 0; clone->str = 0;
     rb_reg_initialize(clone, RREGEXP(re)->str, RREGEXP(re)->len,
 		      rb_reg_options(re));
-    return (VALUE)re;
+    return (VALUE)clone;
 }
 
 VALUE
