@@ -2,8 +2,8 @@
 
   ruby.c -
 
-  $Author: eban $
-  $Date: 2001/09/03 09:59:57 $
+  $Author: nobu $
+  $Date: 2001/12/17 08:13:17 $
   created at: Tue Aug 10 12:47:31 JST 1993
 
   Copyright (C) 1993-2001 Yukihiro Matsumoto
@@ -284,8 +284,8 @@ ruby_init_loadpath()
 struct req_list {
     char *name;
     struct req_list *next;
-} req_list_head;
-struct req_list *req_list_last = &req_list_head;
+};
+static struct req_list req_list_head, *req_list_last = &req_list_head;
 
 static void
 add_modules(mod)
@@ -303,7 +303,7 @@ add_modules(mod)
 
 extern void Init_ext _((void));
 
-void
+static void
 require_libraries()
 {
     extern NODE *ruby_eval_tree;
@@ -597,8 +597,10 @@ proc_options(argc, argv)
 	    s++;
 	    if (strcmp("copyright", s) == 0)
 		copyright = 1;
-	    else if (strcmp("debug", s) == 0)
-		ruby_debug = 1;
+	    else if (strcmp("debug", s) == 0) {
+		ruby_debug = Qtrue;
+                ruby_verbose = Qtrue;
+            }
 	    else if (strcmp("version", s) == 0)
 		version = 1;
 	    else if (strcmp("verbose", s) == 0) {
@@ -895,6 +897,7 @@ set_arg0(val, id)
       j.pst_command = s;
       pstat(PSTAT_SETCMD, j, i, 0, 0);
     }
+    rb_progname = rb_tainted_str_new(s, i);
 #elif defined(HAVE_SETPROCTITLE)
     setproctitle("%.*s", i, s);
     rb_progname = rb_tainted_str_new(s, i);
@@ -927,7 +930,6 @@ set_arg0(val, id)
     }
     rb_progname = rb_tainted_str_new2(origargv[0]);
 #endif
-    rb_progname = rb_tainted_str_new(s, i);
 }
 
 void
