@@ -2,16 +2,16 @@
 
   rubyio.h -
 
-  $Author: matz $
-  $Date: 2000/11/10 07:16:51 $
+  $Author: nobu $
+  $Date: 2001/12/17 08:13:17 $
   created at: Fri Nov 12 16:47:09 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#ifndef IO_H
-#define IO_H
+#ifndef RUBYIO_H
+#define RUBYIO_H
 
 #include <stdio.h>
 #include <errno.h>
