@@ -2,8 +2,8 @@
 #
 #               biorhythm.rb - 
 #                       $Release Version: $
-#                       $Revision: 1.2 $
-#                       $Date: 1999/08/13 05:45:19 $
+#                       $Revision: 1.5 $
+#                       $Date: 2000/07/31 09:21:25 $
 #                       by Yasuo OHBA(STAFS Development Room)
 #
 # --
@@ -11,9 +11,24 @@
 #       
 #
 
+# probably based on:
+#
+# Newsgroups: comp.sources.misc,de.comp.sources.os9
+# From: fkk@stasys.sta.sub.org (Frank Kaefer)
+# Subject: v41i126:  br - Biorhythm v3.0, Part01/01
+# Message-ID: <1994Feb1.070616.15982@sparky.sterling.com>
+# Sender: kent@sparky.sterling.com (Kent Landfield)
+# Organization: Sterling Software
+# Date: Tue, 1 Feb 1994 07:06:16 GMT
+#
+# Posting-number: Volume 41, Issue 126
+# Archive-name: br/part01
+# Environment: basic, dos, os9
+
 include Math
 require "date.rb"
 require "parsearg.rb"
+require "parsedate.rb"
 
 def usage()
   print "Usage:\n"
@@ -35,29 +50,37 @@ def printHeader(y, m, d, p, w)
 end
 
 def getPosition(z)
-  pi = 3.14159265
+  pi = Math::PI
   phys = (50.0 * (1.0 + sin((z / 23.0 - (z / 23)) * 360.0 * pi / 180.0))).to_i
   emot = (50.0 * (1.0 + sin((z / 28.0 - (z / 28)) * 360.0 * pi / 180.0))).to_i
   geist =(50.0 * (1.0 + sin((z / 33.0 - (z / 33)) * 360.0 * pi / 180.0))).to_i
   return phys, emot, geist
 end
 
+def parsedate(s)
+  ParseDate::parsedate(s).indexes(0, 1, 2)
+end
+
+def name_of_week(date)
+  Date::DAYNAMES[date.wday]
+end
+
 #
 # main program
 #
 parseArgs(0, nil, "vg", "D:", "sdate", "date:", "birthday:", "days:")
 
 if $OPT_D
-  dd = Date.new(Time.now.strftime("%Y%m%d"))
-  bd = Date.new($OPT_D)
+  dd = Date.today
+  bd = Date.new(*parsedate($OPT_D))
   ausgabeart = "g"
 else
   if $OPT_birthday
-    bd = Date.new($OPT_birthday)
+    bd = Date.new(*parsedate($OPT_birthday))
   else
     printf(STDERR, "Birthday                      (YYYYMMDD) : ")
     if (si = STDIN.gets.chop) != ""
-      bd = Date.new(si)
+      bd = Date.new(*parsedate(si))
     end
   end
   if !bd
@@ -66,17 +89,17 @@ else
   end
   
   if $OPT_sdate
-    dd = Date.new(Time.now.strftime("%Y%m%d"))
+    dd = Date.today
   elsif $OPT_date
-    dd = Date.new($OPT_date)
+    dd = Date.new(*parsedate($OPT_date))
   else
     printf(STDERR, "Date        [<RETURN> for Systemdate] (YYYYMMDD) : ")
     if (si = STDIN.gets.chop) != ""
-      dd = Date.new(si)
+      dd = Date.new(*parsedate(si))
     end
   end
   if !dd
-    dd = Date.new(Time.now.strftime("%Y%m%d"))
+    dd = Date.today
   end
 
   if $OPT_v
@@ -89,10 +112,10 @@ else
   end
 end
 if (ausgabeart == "v")
-  printHeader(bd.year, bd.month, bd.day, dd.period - bd.period, bd.name_of_week)
+  printHeader(bd.year, bd.month, bd.day, dd - bd, name_of_week(bd))
   print "\n"
   
-  phys, emot, geist = getPosition(dd.period - bd.period)
+  phys, emot, geist = getPosition(dd - bd)
   printf "Biorhythm:   %04d.%02d.%02d\n", dd.year, dd.month, dd.day
   printf "Physical:    %d%%\n", phys
   printf "Emotional:   %d%%\n", emot
@@ -113,13 +136,13 @@ else
     end
   end
 
-  printHeader(bd.year, bd.month, bd.day, dd.period - bd.period, bd.name_of_week)
+  printHeader(bd.year, bd.month, bd.day, dd - bd, name_of_week(bd))
   print "                     P=physical, E=emotional, M=mental\n"
   print "             -------------------------+-------------------------\n"
   print "                     Bad Condition    |    Good Condition\n"
   print "             -------------------------+-------------------------\n"
   
-  for z in (dd.period - bd.period)..(dd.period - bd.period + display_period)
+  for z in (dd - bd)..(dd - bd + display_period)
     phys, emot, geist = getPosition(z)
     
     printf "%04d.%02d.%02d : ", dd.year, dd.month, dd.day
