@@ -3,7 +3,7 @@
   string.c -
 
   $Author: matz $
-  $Date: 2001/09/06 07:47:46 $
+  $Date: 2001/12/04 04:53:31 $
   created at: Mon Aug  9 17:12:58 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -211,6 +211,8 @@ rb_str_dup(str)
     }
     else if (RSTRING(str)->orig) {
 	str2 = rb_str_new3(RSTRING(str)->orig);
+	FL_UNSET(str2, FL_TAINT);
+	OBJ_INFECT(str2, str);
     }
     else {
 	str2 = rb_str_new3(rb_str_new4(str));
@@ -306,9 +308,8 @@ rb_str_times(str, times)
     }
     RSTRING(str2)->ptr[RSTRING(str2)->len] = '\0';
 
-    if (OBJ_TAINTED(str)) {
-	OBJ_TAINT(str2);
-    }
+    OBJ_INFECT(str2, str);
+    RBASIC(str2)->klass = rb_obj_class(str);
 
     return str2;
 }
@@ -354,6 +355,7 @@ rb_str_substr(str, beg, len)
     if (len == 0) return rb_str_new(0,0);
 
     str2 = rb_str_new(RSTRING(str)->ptr+beg, len);
+    RBASIC(str2)->klass = rb_obj_class(str);
     if (OBJ_TAINTED(str)) OBJ_TAINT(str2);
 
     return str2;
@@ -566,8 +568,10 @@ rb_str_equal(str1, str2)
     VALUE str1, str2;
 {
     if (str1 == str2) return Qtrue;
-    if (TYPE(str2) != T_STRING)
-	return Qfalse;
+    if (TYPE(str2) != T_STRING) {
+	str2 = rb_check_convert_type(str2, T_STRING, "String", "to_str");
+	if (NIL_P(str2)) return Qfalse;
+    }
 
     if (RSTRING(str1)->len == RSTRING(str2)->len
 	&& rb_str_cmp(str1, str2) == 0) {
@@ -807,6 +811,7 @@ rb_str_succ(orig)
     int n = 0;
 
     str = rb_str_new(RSTRING(orig)->ptr, RSTRING(orig)->len);
+    RBASIC(str)->klass = rb_obj_class(orig);
     OBJ_INFECT(str, orig);
     if (RSTRING(str)->len == 0) return str;
 
@@ -823,12 +828,12 @@ rb_str_succ(orig)
 	sbeg = RSTRING(str)->ptr; s = sbeg + RSTRING(str)->len - 1;
 	c = '\001';
 	while (sbeg <= s) {
-	    *s += 1;
-	    if (*s-- != 0) break;
+           if ((*s += 1) != 0) break;
+           s--;
 	}
     }
     if (s < sbeg) {
-	REALLOC_N(RSTRING(str)->ptr, char, RSTRING(str)->len + 1);
+	REALLOC_N(RSTRING(str)->ptr, char, RSTRING(str)->len + 2);
 	s = RSTRING(str)->ptr + n;
 	memmove(s+1, s, RSTRING(str)->len - n);
 	*s = c;
@@ -986,12 +991,15 @@ rb_str_aset(str, indx, val)
       case T_FIXNUM:
       num_index:
 	idx = NUM2INT(indx);
-	if (idx < 0) {
-	    idx += RSTRING(str)->len;
-	}
-	if (idx < 0 || RSTRING(str)->len <= idx) {
+	if (RSTRING(str)->len <= idx) {
+	  out_of_range:
 	    rb_raise(rb_eIndexError, "index %d out of string", idx);
 	}
+	if (idx < 0) {
+	    if (-idx > RSTRING(str)->len)
+		goto out_of_range;
+	    idx += RSTRING(str)->len;
+	}
 	if (FIXNUM_P(val)) {
 	    if (RSTRING(str)->len == idx) {
 		RSTRING(str)->len += 1;
@@ -1051,15 +1059,16 @@ rb_str_aset_m(argc, argv, str)
 	beg = NUM2INT(argv[0]);
 	len = NUM2INT(argv[1]);
 	if (len < 0) rb_raise(rb_eIndexError, "negative length %d", len);
-	if (beg < 0) {
-	    beg += RSTRING(str)->len;
-	}
-	if (beg < 0 || RSTRING(str)->len < beg) {
-	    if (beg < 0) {
-		beg -= RSTRING(str)->len;
-	    }
+	if (RSTRING(str)->len < beg) {
+	  out_of_range:
 	    rb_raise(rb_eIndexError, "index %d out of string", beg);
 	}
+	if (beg < 0) {
+	    if (-beg > RSTRING(str)->len) {
+		goto out_of_range;
+	    }
+	    beg += RSTRING(str)->len;
+	}
 	if (beg + len > RSTRING(str)->len) {
 	    len = RSTRING(str)->len - beg;
 	}
@@ -1288,6 +1297,7 @@ str_gsub(argc, argv, str, bang)
 	NEWOBJ(dup, struct RString);
 	OBJSETUP(dup, rb_cString, T_STRING);
 	OBJ_INFECT(dup, str);
+	RBASIC(dup)->klass = rb_obj_class(str);
 	str = (VALUE)dup;
 	dup->orig = 0;
     }
@@ -1432,6 +1442,8 @@ rb_str_reverse(str)
     while (e >= s) {
 	*p++ = *e--;
     }
+    OBJ_INFECT(obj, str);
+    RBASIC(obj)->klass = rb_obj_class(str);
 
     return obj;
 }
@@ -1489,9 +1501,7 @@ VALUE
 rb_str_inspect(str)
     VALUE str;
 {
-    long len;
     char *p, *pend;
-    char *q, *qend;
     VALUE result = rb_str_new2("\"");
     char s[5];
 
@@ -2718,6 +2728,7 @@ rb_str_ljust(str, w)
 
     if (width < 0 || RSTRING(str)->len >= width) return str;
     res = rb_str_new(0, width);
+    RBASIC(res)->klass = rb_obj_class(str);
     memcpy(RSTRING(res)->ptr, RSTRING(str)->ptr, RSTRING(str)->len);
     p = RSTRING(res)->ptr + RSTRING(str)->len; pend = RSTRING(res)->ptr + width;
     while (p < pend) {
@@ -2738,6 +2749,7 @@ rb_str_rjust(str, w)
 
     if (width < 0 || RSTRING(str)->len >= width) return str;
     res = rb_str_new(0, width);
+    RBASIC(res)->klass = rb_obj_class(str);
     p = RSTRING(res)->ptr; pend = p + width - RSTRING(str)->len;
     while (p < pend) {
 	*p++ = ' ';
@@ -2759,6 +2771,7 @@ rb_str_center(str, w)
 
     if (width < 0 || RSTRING(str)->len >= width) return str;
     res = rb_str_new(0, width);
+    RBASIC(res)->klass = rb_obj_class(str);
     n = (width - RSTRING(str)->len)/2;
     p = RSTRING(res)->ptr; pend = p + n;
     while (p < pend) {
