@@ -3,7 +3,7 @@
   time.c -
 
   $Author: matz $
-  $Date: 2001/05/15 08:49:21 $
+  $Date: 2001/12/13 08:12:59 $
   created at: Tue Dec 28 14:31:59 JST 1993
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -87,6 +87,14 @@ time_new_internal(klass, sec, usec)
     VALUE obj;
     struct time_object *tobj;
 
+    if (usec >= 1000000) {	/* usec overflow */
+	sec += usec / 1000000;
+	usec %= 1000000;
+    }
+    while (usec < 0) {		/* usec underflow */
+	sec--;
+	usec += 1000000;
+    }
     if (sec < 0 || (sec == 0 && usec < 0))
 	rb_raise(rb_eArgError, "time must be positive");
 
@@ -459,7 +467,13 @@ time_cmp(time1, time2)
     switch (TYPE(time2)) {
       case T_FIXNUM:
 	i = FIX2LONG(time2);
-	if (tobj1->tv.tv_sec == i) return INT2FIX(0);
+	if (tobj1->tv.tv_sec == i) {
+	    if (tobj1->tv.tv_usec == 0)
+		return INT2FIX(0);
+	    if (tobj1->tv.tv_usec > 0)
+		return INT2FIX(1);
+	    return INT2FIX(-1);
+	}
 	if (tobj1->tv.tv_sec > i) return INT2FIX(1);
 	return INT2FIX(-1);
 	
@@ -486,8 +500,22 @@ time_cmp(time1, time2)
 	if (tobj1->tv.tv_sec > tobj2->tv.tv_sec) return INT2FIX(1);
 	return INT2FIX(-1);
     }
+    if (TYPE(time2) == T_BIGNUM) {
+	double a = (double)tobj1->tv.tv_sec+(double)tobj1->tv.tv_usec/1e6;
+	double b = rb_big2dbl(time2);
+
+	if (a == b) return INT2FIX(0);
+	if (a > b) return INT2FIX(1);
+	if (a < b) return INT2FIX(-1);
+    }
     i = NUM2LONG(time2);
-    if (tobj1->tv.tv_sec == i) return INT2FIX(0);
+    if (tobj1->tv.tv_sec == i) {
+	if (tobj1->tv.tv_usec == 0)
+	    return INT2FIX(0);
+	if (tobj1->tv.tv_usec > 0)
+	    return INT2FIX(1);
+	return INT2FIX(-1);
+    }
     if (tobj1->tv.tv_sec > i) return INT2FIX(1);
     return INT2FIX(-1);
 }
@@ -564,10 +592,13 @@ time_localtime(time)
     time_t t;
 
     GetTimeval(time, tobj);
-    if (tobj->tm_got && !tobj->gmt) {
-	return time;
+    if (!tobj->gmt) {
+	if (tobj->tm_got)
+	    return time;
+    }
+    else {
+	time_modify(time);
     }
-    time_modify(time);
     t = tobj->tv.tv_sec;
     tm_tmp = localtime(&t);
     tobj->tm = *tm_tmp;
@@ -585,10 +616,13 @@ time_gmtime(time)
     time_t t;
 
     GetTimeval(time, tobj);
-    if (tobj->tm_got && tobj->gmt) {
-	return time;
+    if (tobj->gmt) {
+	if (tobj->tm_got)
+	    return time;
+    }
+    else {
+	time_modify(time);
     }
-    time_modify(time);
     t = tobj->tv.tv_sec;
     tm_tmp = gmtime(&t);
     tobj->tm = *tm_tmp;
@@ -662,14 +696,6 @@ time_plus(time1, time2)
     usec = tobj->tv.tv_usec + (time_t)((f - (double)sec)*1e6);
     sec = tobj->tv.tv_sec + sec;
 
-    if (usec >= 1000000) {	/* usec overflow */
-	sec++;
-	usec -= 1000000;
-    }
-    if (usec < 0) {		/* usec underflow */
-	sec--;
-	usec += 1000000;
-    }
     time2 = rb_time_new(sec, usec);
     if (tobj->gmt) {
 	GetTimeval(time2, tobj);
@@ -703,14 +729,6 @@ time_minus(time1, time2)
 	sec = tobj->tv.tv_sec - sec;
     }
 
-    if (usec >= 1000000) {	/* usec overflow */
-	sec++;
-	usec -= 1000000;
-    }
-    if (usec < 0) {		/* usec underflow */
-	sec--;
-	usec += 1000000;
-    }
     time2 = rb_time_new(sec, usec);
     if (tobj->gmt) {
 	GetTimeval(time2, tobj);
@@ -910,7 +928,7 @@ rb_strftime(buf, format, time)
 	 * if the buffer is 1024 times bigger than the length of the
 	 * format string, it's not failing for lack of room.
 	 */
-	if (len > 0 || len >= 1024 * flen) return len;
+	if (len > 0 || size >= 1024 * flen) return len;
 	free(*buf);
     }
     /* not reached */
@@ -946,7 +964,10 @@ time_strftime(time, format)
 	    p += strlen(p) + 1;
 	    if (p <= pe)
 		rb_str_cat(str, "\0", 1);
-	    if (len > SMALLBUF) free(buf);
+	    if (buf != buffer) {
+		free(buf);
+		buf = buffer;
+	    }
 	}
 	return str;
     }
