@@ -2,8 +2,8 @@
 
   util.h -
 
-  $Author: matz $
-  $Date: 2002/12/02 18:19:04 $
+  $Author: nobu $
+  $Date: 2001/12/21 07:20:00 $
   created at: Thu Mar  9 11:55:53 JST 1995
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -58,7 +58,4 @@ char *ruby_strdup _((const char*));
 #undef strdup
 #define strdup(s) ruby_strdup((s))
 
-double ruby_strtod _((const char*, char **));
-#define strtod(s,e) ruby_strtod((s),(e))
-
 #endif /* UTIL_H */
