@@ -3,7 +3,7 @@
   variable.c -
 
   $Author: matz $
-  $Date: 2001/08/23 06:00:31 $
+  $Date: 2001/12/21 09:20:54 $
   created at: Tue Apr 19 23:55:15 JST 1994
 
   Copyright (C) 1993-2000 Yukihiro Matsumoto
@@ -129,6 +129,9 @@ find_class_path(klass)
 	st_foreach(rb_class_tbl, fc_i, &arg);
     }
     if (arg.path) {
+	if (!ROBJECT(klass)->iv_tbl) {
+	    ROBJECT(klass)->iv_tbl = st_init_numtable();
+	} 
 	st_insert(ROBJECT(klass)->iv_tbl,rb_intern("__classpath__"),arg.path);
 	return arg.path;
     }
@@ -142,13 +145,10 @@ classname(klass)
     VALUE path = Qnil;
     ID classpath = rb_intern("__classpath__");
 
-    while (TYPE(klass) == T_ICLASS || FL_TEST(klass, FL_SINGLETON)) {
-	klass = (VALUE)RCLASS(klass)->super;
-    }
+    klass = rb_class_real(klass);
     if (!klass) klass = rb_cObject;
-    if (!ROBJECT(klass)->iv_tbl)
-	ROBJECT(klass)->iv_tbl = st_init_numtable();
-    else if (!st_lookup(ROBJECT(klass)->iv_tbl, classpath, &path)) {
+    if (ROBJECT(klass)->iv_tbl &&
+	!st_lookup(ROBJECT(klass)->iv_tbl, classpath, &path)) {
 	ID classid = rb_intern("__classid__");
 
 	if (st_lookup(ROBJECT(klass)->iv_tbl, classid, &path)) {
@@ -395,7 +395,7 @@ var_getter(id, var)
     ID id;
     VALUE *var;
 {
-    if (!var || !*var) return Qnil;
+    if (!var) return Qnil;
     return *var;
 }
 
@@ -560,7 +560,9 @@ remove_trace(entry)
 	    trace->next = next->next;
 	    free(next);
 	}
-	trace = next;
+	else {
+	    trace = next;
+	}
     }
     entry->trace = t.next;
 }
@@ -592,7 +594,6 @@ rb_f_untrace_var(argc, argv)
 	    trace->removed = 1;
 	    trace = next;
 	}
-	entry->trace = 0;
 
 	if (!entry->block_trace) remove_trace(entry);
 	return ary;
@@ -1378,6 +1379,7 @@ rb_cvar_set(klass, id, val)
     tmp = klass;
     while (tmp) {
 	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,0)) {
+	    if (OBJ_FROZEN(tmp)) rb_error_frozen("class/module");
 	    if (!OBJ_TAINTED(tmp) && rb_safe_level() >= 4)
 		rb_raise(rb_eSecurityError, "Insecure: can't modify class variable");
 	    st_insert(RCLASS(tmp)->iv_tbl,id,val);
@@ -1401,6 +1403,7 @@ rb_cvar_declare(klass, id, val)
     tmp = klass;
     while (tmp) {
 	if (RCLASS(tmp)->iv_tbl && st_lookup(RCLASS(tmp)->iv_tbl,id,0)) {
+	    if (OBJ_FROZEN(tmp)) rb_error_frozen("class/module");
 	    if (!OBJ_TAINTED(tmp) && rb_safe_level() >= 4)
 		rb_raise(rb_eSecurityError, "Insecure: can't modify class variable");
 	    st_insert(RCLASS(tmp)->iv_tbl,id,val);
